@@ -1,10 +1,12 @@
 import re
-from typing import DefaultDict, Dict, Union, TextIO, List, Any
+from typing import DefaultDict, Dict, Union, TextIO, List, Any, Generator, Tuple
 from sys import argv
+from os import getenv, name as os_name
+from io import StringIO, BytesIO
 from collections import defaultdict
 from pathlib import Path
 
-__version__ = '2.0.6'
+__version__ = '2.0.8'
 GLOBALS_KEY = '_globals'
 
 
@@ -27,7 +29,7 @@ def argv_to_dict(args: List[str], aliases: Dict[str, str] = None) -> DefaultDict
     result = defaultdict(list)
     for a in args:
         # arguments are prefixed with -, -- or / - no distinction for long names, so --h or -help would be valid
-        if a[0] in '-/':
+        if len(a) > 0 and a[0] in '-/':
             if len(a) == 1:
                 raise SyntaxError(f'Syntax error in argument: {a}')
             key = a[2:] if a[:2] == '--' else a[1:]
@@ -303,15 +305,24 @@ class DictConfig(dict):
         return node
 
     @classmethod
-    def from_file(cls, file: Union[TextIO, str] = None, file_type: str = None,
-                  parse_args: bool = True, require_file: bool = True, load_kwargs: dict = None, **kwargs):
+    def _file_from_url(cls, url, url_kwargs):
+        from urllib import request
+        from urllib.parse import urlparse
+        return request.urlopen(url, **url_kwargs), Path(urlparse(url).path).name
+
+    @classmethod
+    def from_file(cls, file: Union[TextIO, BytesIO, str] = None, file_type: str = None,
+                  parse_args: bool = True, require_file: bool = True, url_kwargs: dict = None,
+                  load_kwargs: dict = None, **kwargs):
         """
         Factory method that loads a Config from file and initialises a new instance with the contents.
         Currently only supports .json and .pickle
         :param file: existing configuration filename (or open file pointer)
-        :param file_type: either a file extension ('json', etc.) or None (to use the suffix of `filename`)
+        :param file_type: either a file extension ('json', etc.) or None (will use the suffix of `filename`)
         :param parse_args: whether to parse command line arguments to override 'cfg', list of args to override
+               Note: still requires .update_from_arguments() to be called, only required here for -cfg override
         :param require_file: whether a configuration file is required (otherwise command line args only is accepted)
+        :param url_kwargs: a dictionary containing keyword arguments to pass to a request urlopen method
         :param load_kwargs: a dictionary containing keyword arguments to pass to the format-specific load method
         :param kwargs: additional keyword arguments passed to Config constructor
         :return: initialised DictConfig instance
@@ -331,47 +342,71 @@ class DictConfig(dict):
                 cfg = cls()
                 filename = None
         else:
+            # determine filename and whether a file needs to be opened
+            open_file = False
             if isinstance(file, str) or isinstance(file, Path):
                 file = str(file)
-                if not Path(file).is_file():
-                    raise FileExistsError(f'Config file {file} not found.')
-                filename = file
+                try:
+                    if Path(file).is_file():
+                        filename = file
+                        open_file = True
+                    else:
+                        raise OSError
+                except OSError:
+                    try:
+                        # at this point, file is neither a handle nor a valid file name, try it as a URL
+                        file, filename = cls._file_from_url(file, url_kwargs if url_kwargs is not None else {})
+                    except IOError:
+                        # at this point, file is neither a handle, a valid file name or a valid URL
+                        raise FileExistsError(f'Config file {file} not found.')
             else:
                 try:
                     filename = file.name
                 except AttributeError:
                     filename = None
+
+            # determine file type from name, if not specified
             if file_type is None:
                 if filename is None:
                     raise SyntaxError('File without name requires file_type to be specified')
                 file_type = Path(filename).suffix.lower()[1:]
-            if load_kwargs is None:
-                load_kwargs = {}
-            if file_type == 'json':
-                import json
-                if isinstance(file, str):
-                    with open(filename, 'r') as f:
-                        cfg = cls(json.load(f, **load_kwargs), **kwargs)
+
+            try:
+                # open file if needed at this point
+                if open_file:
+                    if file_type in ['json', 'xml']:
+                        file = open(filename, 'r')
+                    elif file_type == 'pickle':
+                        file = open(filename, 'rb')
                 else:
+                    # if file is either a file object opened with 'b', or not a descendent of StringIO, wrap it
+                    if file_type in ['json', 'xml'] and (
+                            (hasattr(file, 'mode') and file.mode == 'b') or not isinstance(file, StringIO)):
+                        file = StringIO(file.read().decode())
+
+                # based on file_type, obtain cfg from the file handle
+                if load_kwargs is None:
+                    load_kwargs = {}
+                if file_type == 'json':
+                    import json
                     cfg = cls(json.load(file, **load_kwargs), **kwargs)
-            elif file_type == 'pickle':
-                import pickle
-                if isinstance(file, str):
-                    with open(filename, 'rb') as f:
-                        cfg = pickle.load(f, **load_kwargs)
-                else:
+                elif file_type == 'pickle':
+                    import pickle
                     cfg = pickle.load(file, **load_kwargs)
-            elif file_type == 'xml':
-                from lxml import etree
-                root = etree.parse(file, **load_kwargs).getroot()
-                cfg = cls._xml2cfg(root, **kwargs)
+                elif file_type == 'xml':
+                    from lxml import etree
+                    root = etree.parse(file, **load_kwargs).getroot()
+                    cfg = cls._xml2cfg(root, **kwargs)
+            finally:
+                if open_file:
+                    file.close()
 
         cfg.filename = filename
         cfg.arguments = args
 
         return cfg
 
-    def save(self, file: Union[TextIO, str] = None, file_type: str = None, include_globals: bool = True,
+    def save(self, file: Union[TextIO, BytesIO, str] = None, file_type: str = None, include_globals: bool = True,
              include_from_arguments: bool = True, **kwargs):
         """
         Save the config to a file of specified type
@@ -423,35 +458,36 @@ class DictConfig(dict):
                                  [] if include_from_arguments else self.from_arguments)
             etree.ElementTree(root).write(file, encoding='utf-8', xml_declaration=True, **kwargs)
 
+    def _set_value(self, k, v):
+        # existing key?
+        if k in self:
+            # for bool, check specific non-True values
+            if isinstance(self[k], bool):
+                self[k] = v.lower() not in ['0', 'false']
+            else:
+                # for other types, cast to type of existing key
+                t = type(self[k])
+                try:
+                    self[k] = t(v)
+                except ValueError:
+                    raise SyntaxError(f'Cannot cast {v} to {t} from arguments')
+        else:
+            # define new key
+            self.from_arguments.append(k)
+            self[k] = v
+
     def update_from_arguments(self, args: Union[Dict[str, list], list] = None, aliases: Dict[str, str] = None,
                               force_update=False):
         """
         Update the Config with values parsed from the command line arguments. Overwriting values will be cast to the
         same type as the overwritten value, all other values will remain str. Parameters with no value will be set to
-        True.
+        True. If the config was created with `.from_file()` and `parse_args` was not False, it will use the arguments
+        available at the time.
         :param args: a dictionary of arguments, like the one returned by argv_to_dict, or a list like sys.argv
         :param aliases: a dictionary with mappings passed to argv_to_dict, if args is a dictionary
-        :param force_update: whether to interpret args, even if self.arguments is not None
+        :param force_update: whether to interpret args, even if self.arguments is not None (e.g. from .from_file())
         :return: self
         """
-        def set_value(k, v):
-            # existing key?
-            if k in self:
-                # for bool, check specific non-True values
-                if isinstance(self[k], bool):
-                    self[k] = v.lower() not in ['0', 'false']
-                else:
-                    # for other types, cast to type of existing key
-                    t = type(self[k])
-                    try:
-                        self[k] = t(v)
-                    except ValueError:
-                        raise SyntaxError(f'Cannot cast {v} to {t}')
-            else:
-                # define new key
-                self.from_arguments.append(k)
-                self[k] = v
-
         if args is None:
             args = argv
         if isinstance(args, list):
@@ -467,14 +503,89 @@ class DictConfig(dict):
             else:
                 # unpack single value lists
                 if len(value) == 1:
-                    set_value(key, value[0])
+                    self._set_value(key, value[0])
                 else:
                     if not value:
                         # set to True for empty value
-                        set_value(key, True)
+                        self._set_value(key, True)
                     else:
                         # set as list for multi-value
-                        set_value(key, value)
+                        self._set_value(key, value)
+
+        # allow chaining
+        return self
+
+    def recursive_keys_tuples(self) -> Generator[Tuple[str, Tuple[str]], None, None]:
+        for key, value in self.items():
+            yield key, (key,)
+            if isinstance(value, DictConfig):
+                for compound_sub_key, sub_key in value.recursive_keys_tuples():
+                    yield f'{key}.{".".join(sub_key)}', tuple([key, *sub_key])
+
+    def recursive_keys(self) -> Dict[str, Tuple[str]]:
+        """
+        a generator that yields every key in the DictConfig as a dictionary of compound key: key as a tuple
+        :return: Generator[str, None, bool] containing all valid keys for self
+        """
+        return dict(self.recursive_keys_tuples())
+
+    @staticmethod
+    def _case_safe(key, keys):
+        # only on Windows, replace the key with a matching key ignoring case
+        if os_name == 'nt':
+            for k in keys:
+                if k.lower() == key.lower():
+                    return k
+        return key
+
+    def update_from_environment(self, env_vars: list = None, exclude_vars: List = None):
+        """
+        Update the Config with values from the system environment. If no specific `env_vars` are provided, any value
+        in the Config 'shadowed' by an environment variable will get updated.
+        :param env_vars: specific variables to update or add from the environment, or None to update any pre-defined
+        :param exclude_vars: variables that should not be updated (typically if env_vars is None)
+        :return: self
+        """
+        # TODO: parse environment variables case-insensitive on Windows, currently case sensitive
+        if exclude_vars is None:
+            exclude_vars = []
+        environment = {}
+        recursive_keys = self.recursive_keys()
+        if env_vars is None:
+            for compound_key, key in recursive_keys.items():
+                if getenv(compound_key) is not None and self._case_safe(compound_key, exclude_vars) not in exclude_vars:
+                    environment[key] = getenv(compound_key)
+        else:
+            for key in env_vars:
+                key = self._case_safe(key, recursive_keys)
+                if getenv(key) is not None and self._case_safe(key, exclude_vars) not in exclude_vars:
+                    if key in recursive_keys:
+                        environment[recursive_keys[key]] = getenv(key)
+                    else:
+                        environment[key] = getenv(key)
+                        self[key] = None
+        # perform update with constructed environment (not using compound keys, for simplicity)
+        for keys, value in environment.items():
+            d = self
+            for key in keys[:-1]:
+                d = d[key]
+            key = keys[-1]
+
+            # for bool, check specific non-True values
+            if isinstance(d[key], bool):
+                d[keys[-1]] = value.lower() not in ['0', 'false']
+            else:
+                # for other types, cast to type of existing key, or str if None
+                if d[key] is None:
+                    t = str
+                else:
+                    t = type(d[key])
+                try:
+                    d[keys[-1]] = t(value)
+                except ValueError:
+                    raise SyntaxError(f'Cannot cast {value} to {t} from environment')
+
+        # allow chaining
         return self
 
 
